<html>
    <head>
        <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.6/css/bootstrap.min.css" media="screen" />
    </head>
    <body>
        <div class="row">
            <div class="col-md-12">
                <div class="container">
                    <div class="jumbotron">
                        <h2>Custom Landing Page</h2>
                        <p><a class="btn btn-default" href="{{landing_url}}" target="_blank">Launch Now</a></p>
                    </div>
                </div>
            </div>
        </div> 
    </body>
</html>