<html>
<head>
    <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.6/css/bootstrap.min.css" media="screen" />
    
    <style>
        #6f5499
    </style>    
</head>
<body>
    <div class="row">
        <div class="col-md-12">
            <div class="container">
                <div class="jumbotron">
                    <h2>Authentication Failed</h2>
                    <p>
                        Your {{title}} course was not able to authenticate 
                        with this external component.
                    </p>

                    <p>
                        <small>
                            Please report this issue to your administrator
                        </small>
                    </p>
                </div>
            </div>
        </div>
    </div> 
</body>
</html>
